import jcreek.representation.*;
import jcreek.cke.importexport.xml.*;

import java.io.*;
import java.util.Iterator;

/**
 * Class that runs the CaseParser and takes care of verification and paths.
 * @author jano
 */
public class CaseImporter {
	private String xmlFile;// = "test.xml";
	private String outputModel;// = "output.km";
	private String inputModel;// = "template2.km";
	private boolean aborted = false;
	private KnowledgeModel km;

	public CaseImporter (String inputModel, String outputModel, String xmlFile) {
		this.xmlFile = xmlFile;
		this.outputModel = outputModel;
		this.inputModel = inputModel;
	}
	
	public void init(){
		if(xmlFile != null){
			String schemaPath = getClass().getClassLoader().getResource("jcreekmodel.xsd").toExternalForm();
			XMLvalidater validator;
			
			validator = new XMLvalidater(xmlFile, schemaPath);
			
			try{
				km = new LocalKnowledgeModel(inputModel);
				System.out.println("**** Knowledge model "+inputModel+" successfully loaded! ****");
			} catch (FileNotFoundException f) {
				System.out.println("File containing knowledge model, "+ inputModel +" was not found!");
			} catch (ClassNotFoundException cls) {
				cls.printStackTrace();
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
			
			if(schemaPath != null && schemaPath.length() > 0){
				if (validator.isValid()) {
					if (aborted) {
						System.out.println("The parsing was aborted!");
						return;
					}
					CaseParser caseParser = new CaseParser(validator.getDocument(),km, false);
					caseParser.startTraverse(); // Start traversing the XML tree
					if (caseParser.isAbortOnFatalErrorsEnabled()&& caseParser.didFatalErrorOccur()) {
						aborted = true;
					}
					// No fatal errors were caught, try to save the model.
					try{
						((LocalKnowledgeModel)km).saveAs(outputModel);
						System.out.println("**** Knowledge model successfully saved as "+outputModel+" ****");
					} catch(Exception ie){
						//ie.printStackTrace();	
						System.out.println("Exception caught during attempt to save the knowledge model as "+outputModel+" !");
					}
				}
				else {
					System.out.println("XML validation failed, please check the syntax of the following statements:");
					System.out.println(validator.getAllFaultMsg());
				}
			}
			else{
				System.out.println(".xsd file was not find in the path specified.");
			}
		}
		else{
			System.out.println("The specified .jcxml file does not exist!");
		}		
	}//end init()
	
	/**
	 * Prints all entities contained in the output model.
	 *
	 */
	public void printEntities(){
		try{
			BufferedWriter out = new BufferedWriter(new FileWriter("entites.txt"));
			out.write("**** "+outputModel+" contains the following entites ****\n");
			Iterator entities = km.getEntityIterator();
			int noOfEntities = 0;
			
			while(entities.hasNext()){
				Entity next = (Entity)entities.next();
			    out.write(next.getName()+"\n");
			    //System.out.println(next.getName());
			    noOfEntities += 1;
			}
			out.write("**** "+noOfEntities+ " entities was written to file ****");
			System.out.println("**** "+noOfEntities+ " entities was written to file ****");
			out.close();
		} catch(IOException ioe) {
			ioe.printStackTrace();
			System.out.println("Exception caught while writing to file!");
		}
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		System.out.println("Usage: CaseImporter InputKnowledgeModel OutputKnowledgeModel InputJCXMLFile");
		CaseImporter importer = new CaseImporter(args[0], args[1], args[2]);
		importer.init();
		importer.printEntities();
	}
}