% Function for creating region/organ names. The regions are labeled
% according to the best matching organ in the database, as given by the
% match_matrix. 
function named_vp = create_region_names(unlabeled_vp, labeled_vp, match_matrix);

fields = fieldnames(unlabeled_vp);

named_vp = unlabeled_vp;

for i = 1:size(fields,1)
    field = cell2mat(fields(i));
    if(strcmp(field(1:3), 'reg'))
        unlabeled_reg = getfield(named_vp, field);
        found = false;
        corresponding_region = match_matrix(2,i-1);
        if(corresponding_region >0)
            %Must add one to the region, to adjust for the viewplane being the
            %first field in the struct. 
            labeled_ref = getfield(labeled_vp, cell2mat(fields(corresponding_region+1)));
            if(isfield(labeled_ref,'name'))
                labeled_name = getfield(labeled_ref, 'name');
                unlabeled_reg.name = labeled_name;
            else
                temp = unlabeled_reg;
                named_vp = rmfield(named_vp, field);
                labeled_name = getfield(labeled_ref, 'finding');
                temp.name = labeled_name;
                named_vp = setfield(named_vp, 'finding', temp);
                found = true;
            end
        else
            unlabeled_reg.name = 'unknown';
        end
        if(~found)
            named_vp = setfield(named_vp, field, unlabeled_reg);
        end
    end
end