%EXTRACT_REGIONS Extract regions from a segmented image.
%reg = extract_regions(filename, ,no_of_regions, p, q) extract no_of_regions
%from the image specified by filename. The descriptors are also computeed and returned. 

function viewplane = extract_regions(filename, type_of_viewplane);
if nargin ~= 2
    error('ErrorTests:convertTest', 'missing parameters \n Usage: extract_regions(filename, type_of_viewplane)\n filename: the path to the segmented file from which you want to extract the regions \n type_of_viewplane: the viewplane');
end     

tic
p = 3;
q = 3;

image = imread(filename);
image_regions = abs(image(:,:,1)-image(:,:,3))>50;
%Give each region a unique label for further processing
[paths, no_of_regions] = iterative_paths(image_regions);
figure, imshow(paths, []);
regs = zeros(1,no_of_regions);
for i = 1:no_of_regions
    regs(i) = i;
end
[dummy, circums] = find_area(paths, regs);
largest_region = find(circums == max(circums));

if(strcmpi(type_of_viewplane, 'mad'))
    regs = find(regs~=largest_region);
    paths_clear = paths;
    paths = imfill(paths);
    [area_large, dummy] = find_area(paths, largest_region); %Finds the area of the largest region
    for y = 1:size(paths,1)
        for x = 1:size(paths,2)
            if(paths_clear(y,x)==largest_region+1)
                paths_clear(y,x)=0;
            end
        end
    end
end

[areas, dummy] = find_area(imfill(paths_clear), regs);
areas(largest_region) = area_large;
areas = areas./(size(image,1)*size(image,2));
circums = circums./(size(image,1)*size(image,2));
if(strncmpi(type_of_viewplane, 'mad', 3))
    mad = compute_mad(paths, find(areas()==max(areas)));
end
centroids = find_centroids(paths, no_of_regions);

moments = find_moments(paths, p,q, no_of_regions);

central_moments = find_central_moments(paths, p, q, centroids, no_of_regions);

unscaled_central = central_moments;
for i = 1:size(central_moments,1)
    for p1 = 0:p
        for q1 = 0:q
            my00 = central_moments(i,1)^(0.5*(p1+q1)+1);
            x_pos = 1 + (p1*4)+q1;
            if(x_pos~=-1)
                unscaled_central(i,x_pos) = unscaled_central(i,x_pos)/my00; 
            end
        end
    end 
end
unscaled_central;
invariant_moments = find_invariant_moments(unscaled_central);

%Create a struct to keep all the information contained in the image. 
viewplane = struct('viewplane', type_of_viewplane);

for i = 1:no_of_regions
    regions = struct;
    regions.area = areas(i);
    regions.circumference = circums(i);
    cent = struct;
    cent.y = centroids(1,i);
    cent.x = centroids(2,i);
    regions.centroids = cent;
    regions.compactness = ((circums(i)*circums(i))/areas(i))*(size(image,1)*size(image,2));
    inv = struct;
    inv.m1 = invariant_moments(i,1);
    inv.m2 = invariant_moments(i,2);
    inv.m3 = invariant_moments(i,3);
    inv.m4 = invariant_moments(i,4);
    inv.m5 = invariant_moments(i,5);
    inv.m6 = invariant_moments(i,6);
    inv.m7 = invariant_moments(i,7);
    regions.invariant_moments = inv;
    reg = sprintf('region%d', i);
    viewplane = setfield(viewplane, reg, regions);
end
if(exist('mad')) 
    viewplane.mad = mad; 
end;
toc
