% This function matched two feature vectors, and returns the score between
% the best matchin regions in the two vectors. 
%unlabeled_filename: the name of the xml-file storing the information about the
%regions you want to label. 
%labeled_filename: the name of the xml-file storing information about already
%labeled regions (our organ database). 
%This function also saves the files in the correct CBR file-format.
function matched_regions = match_regions(unlabeled_filename, labeled_filename);

%This vector is used to give weights to the different descriptors.
weight_vector_new = [0.05 0.1 0.3 0.05 0.5 0.2 0.1 0.02 0.02 0.02 0.02];
weight_vector =  [0.05 0.1 0.3 0.05 0.5];

unlabeled_vp = xml_load(unlabeled_filename, 'on');

labeled_vp = xml_load(labeled_filename, 'on');

unlabeled_vp_regions = fieldnames(unlabeled_vp);
labeled_vp_regions = fieldnames(labeled_vp);

%If in the same view plane:
if strcmp(unlabeled_vp.viewplane, labeled_vp.viewplane)
    
    no_of_regions = size(unlabeled_vp_regions, 1)-2;
    matched_regions = zeros(4,no_of_regions);
    
    % Iterates over all the regions in the image we want to label.
    % Starts at field 2 (region 1), since the first field is the name of the
    % viewplane. Ends at the second to last 
    for r = 1:size(unlabeled_vp_regions,1)
        fieldname = cell2mat(unlabeled_vp_regions(r));
        if(strncmp(fieldname, 'reg',3))
            unlabeled_region = getfield(unlabeled_vp, cell2mat(unlabeled_vp_regions(r)));
            unlabeled_region_descriptors = fieldnames(unlabeled_region);

            score = Inf;

            most_similar_region = -1;
            %Iterates over all the regions in the already labeled image, to
            %find the most similar region. 
            for l = 2:size(labeled_vp_regions, 1)


                labeled_region = getfield(labeled_vp, cell2mat(labeled_vp_regions(l)));
                labeled_region_type = class(labeled_region);
                if(strcmp(labeled_region_type, 'struct'))
                    labeled_region_descriptors = fieldnames(labeled_region);

                    %Checks whether the number of descriptors are equal (labeled has
                    %also got the name as a descriptor). 
                    if (size(labeled_region_descriptors,1)-1==size(unlabeled_region_descriptors,1))
                        %Iterates over all the descriptors in each region.
                        temp_score = 0;
                        temp_dist = 0;


                        for f = 1:size(unlabeled_region_descriptors)
                            %Extract the descriptor on which we want to compare, i.e.,
                            %area, circumference, compactness, etc. 
                            current_descriptor = cell2mat(unlabeled_region_descriptors(f));
                            current_unlabeled_descriptor = getfield(unlabeled_region, current_descriptor);
                            current_labeled_descriptor = getfield(labeled_region, current_descriptor);
                            [temp_score2 type]= compute_score(current_unlabeled_descriptor, current_labeled_descriptor);
                            if(type==0)
                                temp_score = temp_score+(temp_score2*weight_vector(f));
                            else
                                temp_score = temp_score+((temp_score2/314)*weight_vector(f));
                                temp_dist = temp_score2;
                            end
                        end
                    else
                        error('Unequal number of descriptors')
                    end
                    if temp_score<score
                        score = temp_score;
                        distance = temp_dist;
                        most_similar_region = l-1;
                    end
                end
            end
            matched_regions(1,r-1) = r-1;
            %Checks whether or not a previous regions has been labeled with the
            %same region
            if(find(matched_regions(2,:)==most_similar_region))
                %We find a better match and discards the previous match
                if(score<matched_regions(3,find(matched_regions(2,:)==most_similar_region)))
                    wrong_region_match = find(matched_regions(2,:)==most_similar_region);
                    matched_regions(2,wrong_region_match) = -1;
                    matched_regions(2,r-1) = most_similar_region;
                %The previous match was the better, and we discard this match
                else
                    matched_regions(2,r-1) = -1;
                end
            else
                matched_regions(2, r-1) = most_similar_region;
            end
            matched_regions(3,r-1) = score;
            matched_regions(4,r-1) = distance;
        end
    end
else
    matched_regions = false;
end

labeled_input = create_region_names(unlabeled_vp, labeled_vp, matched_regions(1:2, :));

create_xml_rep(labeled_input, strcat(unlabeled_filename(1:size(unlabeled_filename,2)-4), '_cbr.xml'));